/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * $Id: DeploymentPlanGenerator.java 4798 2009-08-07 12:30:47Z alitokmen $
 */

package org.ow2.jasmine.monitoring.installer.deploymentplan;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.Properties;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

/**
 * @author Alina Radu
 */
public class DeploymentPlanGenerator {

    /**
    *
    */
    private VelocityEngine ve = null;

    /**
    *
    */
    private VelocityContext context = null;

    /**
    *
    */
    private Template template = null;

    /**
     *
     */
    private static final String DEP_PLAN_M2_NAME_TEMPLATE = "jasmine-monitoring-plan.vm";

    /**
    *
    */
    private static final String DEP_PLAN_URL_NAME_TEMPLATE = "jasmine-monitoring-plan-url.vm";

    /**
    *
    */
    private static final String REPO_NAME_TEMPLATE = "repository.vm";

    /**
    *
    */
    private static final String DEP_PLAN_M2_NAME = "zz_jasmine-monitoring-plan.xml";

    /**
    *
    */
    private static final String REPO_NAME = "repository.xml";

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    /**
     * Constructor
     *
     * @param hasMbeancmd
     * @param hasDB
     * @param hasDrools
     * @param hasEos
     * @param repo
     * @param output
     */
    public DeploymentPlanGenerator(boolean hasMbeancmd, boolean hasDB, boolean hasDrools, boolean hasEos, boolean hasNotif, boolean hasNotifExportJMX, boolean hasNotifExportSNMP, String repo,
            String output) {

        // lines to find the template even within the jar file
        final Properties p = new Properties();
        p.setProperty("resource.loader", "class");
        p.setProperty("class.resource.loader.class",
                "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");

        ve = new VelocityEngine();
        context = new VelocityContext();

        try {
            ve.init(p);
            final StringWriter writer = new StringWriter();

            if (repo.contains("bundles")) {
                template = ve.getTemplate(REPO_NAME_TEMPLATE);
                if(!repo.startsWith("/")){
                    repo = "/" + repo;
                }
                context.put("repo_dir", repo);
                if (template != null) {
                    template.merge(context, writer);
                }
                write(output + File.separator + REPO_NAME);
            }

            if (repo.contains("bundles")) {
                template = ve.getTemplate(DEP_PLAN_URL_NAME_TEMPLATE);
            } else {
                template = ve.getTemplate(DEP_PLAN_M2_NAME_TEMPLATE);
                context.put("repository-ref", repo);
            }

            context.put("mbeancmd", new Boolean(hasMbeancmd));
            context.put("eventstodb", new Boolean(hasDB));
            context.put("drools", new Boolean(hasDrools));
            context.put("eos", new Boolean(hasEos));
            context.put("notif", new Boolean(hasNotif));
            context.put("notif-export-jmx", new Boolean(hasNotifExportJMX));
            context.put("notif-export-snmp", new Boolean(hasNotifExportSNMP));

            if (template != null) {
                template.merge(context, writer);
            }
            write(output + File.separator + DEP_PLAN_M2_NAME);

        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Write generated configuration in the file "filename"
     * 
     * @param filename
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */

    private void write(final String filename) throws Exception {
        final File file = new File(filename);
        if (!file.exists()) {
            file.createNewFile();
        }

        final BufferedWriter out = new BufferedWriter(new FileWriter(filename));
        out.write(generate());
        out.close();
    }

    /**
     * @return
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private String generate() throws Exception {

        final StringWriter writer = new StringWriter();

        if (template != null) {
            template.merge(context, writer);
        }
        return new String(writer.getBuffer());
    }
}
